﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

namespace CableExperiment
{
    public partial class Window1 : Window
    {
        const double MAX_CONTROL_POINT_Y_OFFSET = 100;
        const double CUTOFF_DIFFERENCE = 5;
        double cableLength;
        FrameworkElement elementDragged;
        Point plugPointStart;
        Point mousePointStart;
        PathGeometry shadowCableGeometry = new PathGeometry();
        PathFigure shadowCableFigure = new PathFigure();
        BezierSegment shadowCableSegment = new BezierSegment();

        public Window1()
        {
            InitializeComponent();

            cableLength = GetLength(cableGeometry);
            shadowCableFigure.Segments.Add(shadowCableSegment);
            shadowCableGeometry.Figures.Add(shadowCableFigure);
        }

        protected override void OnMouseLeftButtonDown(MouseButtonEventArgs args)
        {
            if (args.Source == plug1 || args.Source == plug2)
            {
                elementDragged = args.Source as FrameworkElement;

                if (elementDragged == plug1)
                    plugPointStart = cableFigure.StartPoint;
                else
                    plugPointStart = cableSegment.Point3;

                // Define points for shadow geometry
                shadowCableFigure.StartPoint = cableFigure.StartPoint;
                shadowCableSegment.Point1 = cableSegment.Point1;
                shadowCableSegment.Point2 = cableSegment.Point2;
                shadowCableSegment.Point3 = cableSegment.Point3;

                mousePointStart = args.GetPosition(this);
                CaptureMouse();
            }
        }

        protected override void OnMouseMove(MouseEventArgs args)
        {
            Point mousePoint = args.GetPosition(this);

            if (elementDragged == null)
                return;

            // Set new points in shadow geometry
            if (elementDragged == plug1)
            {
                shadowCableFigure.StartPoint = plugPointStart + (mousePoint - mousePointStart);
                shadowCableSegment.Point1 = new Point(shadowCableFigure.StartPoint.X, shadowCableSegment.Point1.Y);
            }

            else if (elementDragged == plug2)
            {
                shadowCableSegment.Point3 = plugPointStart + (mousePoint - mousePointStart);
                shadowCableSegment.Point2 = new Point(shadowCableSegment.Point3.X, shadowCableSegment.Point2.Y);
            }

            // Get a range of Y's for the Bezier control points
            double yLowerBound = MAX_CONTROL_POINT_Y_OFFSET + Math.Max(shadowCableFigure.StartPoint.Y, shadowCableSegment.Point3.Y);
            double yUpperBound = shadowCableFigure.StartPoint.Y + shadowCableSegment.Point3.Y + cableLength;

            if (yLowerBound > yUpperBound)
                return;

            // Find lengths based on two Y's
            double lowerBoundLength = SetNewControlPointsAndGetLength(yLowerBound);
            double upperBoundLength = SetNewControlPointsAndGetLength(yUpperBound);

            if (lowerBoundLength > cableLength || upperBoundLength < cableLength)
                return;

            // Narrow in on cable length
            double yTrial = (yLowerBound + yUpperBound) / 2;

            while (yUpperBound - yLowerBound > CUTOFF_DIFFERENCE)
            {
                double trialLength = SetNewControlPointsAndGetLength(yTrial);

                if (trialLength < cableLength)
                    yLowerBound = yTrial;
                else
                    yUpperBound = yTrial;

                yTrial = (yLowerBound + yUpperBound) / 2;
            }

            SetNewControlPointsAndGetLength(yTrial);

            // Set cable points to shadow points
            cableFigure.StartPoint = shadowCableFigure.StartPoint;
            cableSegment.Point1 = shadowCableSegment.Point1;
            cableSegment.Point2 = shadowCableSegment.Point2;
            cableSegment.Point3 = shadowCableSegment.Point3;
        }

        // Mouse up: Release capture
        protected override void OnMouseLeftButtonUp(MouseButtonEventArgs args)
        {
            elementDragged = null;
            ReleaseMouseCapture();
        }

        // Calculate length of a the trial geometry based on control-point Y values
        double SetNewControlPointsAndGetLength(double y)
        {
            shadowCableSegment.Point1 = new Point(shadowCableFigure.StartPoint.X, y);
            shadowCableSegment.Point2 = new Point(shadowCableSegment.Point3.X, y);
            return GetLength(shadowCableGeometry);
        }

        // Calculate length of a one-figure geometry
        double GetLength(Geometry geometry)
        {
            double length = 0;
            PathGeometry pathGeometry = geometry.GetFlattenedPathGeometry();
            PathFigure pathFigure = pathGeometry.Figures[0];
            PolyLineSegment polyLineSegment = pathFigure.Segments[0] as PolyLineSegment;
            Point ptPrevious = pathFigure.StartPoint;

            foreach (Point pt in polyLineSegment.Points)
            {
                length += (pt - ptPrevious).Length;
                ptPrevious = pt;
            }

            return length;
        }
    }
}
